# File: Killer.ps1
# Syntax: ./Killer.ps1 [-checkEverySec N] processName_1 [processName_2] [processName_3] ...
#
# Check if the given processes exist and try to kill all instances of them.
# The execution can be configured for one time execution or repeated
# (time controlled) to capture also unwanted processes which are started
# later in time.

#----------------------------------------------------------

[int]$script:REPEAT_CHECK_AFTER_SECONDS = 30
[string[]]$script:processesToDelete = @() # empty array of process names

#----------------------------------------------------------

function DisplayUsageAndTerminate
{
    ""
    "Killer.ps1 - commandline syntax"
    ""
    "-checkEverySec <N>"
    ""
    "  repeat the check for unwanted processes when the given time interval"
    "  has elapsed; default time interval is 30 seconds; a time interval"
    "  of 0 means single execution;"
    ""
    "processname_1 processName_2 ..."
    ""
    "  a list of (unwanted) processes to be deleted, the names are separated"
    "  by blanks; you have to specify at least one process name"
    ""
    "-? or -help"
    ""
    "  show these infos and exit from script execution"
    ""
    exit

} # DisplayUsageAndTerminate


#----------------------------------------------------------

# Remark: using "$args" within function would access the arguments
# passed to the function and not the arguments passed to the script
function ReadCommandLineArgs ($commandLineArgs) 
{
    $numArgs = $commandLineArgs.length
    $idx = 0
    while ($idx -lt $numArgs)
    {
        if (($commandLineArgs[$idx] -eq "-?") -or ($commandLineArgs[$idx] -eq "-help"))
        {
            DisplayUsageAndTerminate
        }
        elseif ($commandLineArgs[$idx] -eq "-checkEverySec")
        {
            if ($idx+1 -lt $numArgs)
            {
                $script:REPEAT_CHECK_AFTER_SECONDS = $commandLineArgs[$idx+1]
                $idx += 1
            }
            else # invalid syntax
            {
                DisplayUsageAndTerminate
            }
        }
        else # assume process name
        {
            $script:processesToDelete+= $commandLineArgs[$idx]
        }
        $idx+=1
    }
    if ($script:processesToDelete.length -le 0)
    {
        # at least a single process name has to be specified
        DisplayUsageAndTerminate
    }
    
} # ReadCommandLineArgs

#----------------------------------------------------------

ReadCommandLineArgs $args

[string]$repeatInfo = ""
if ($script:REPEAT_CHECK_AFTER_SECONDS -gt 0)
{
    $repeatInfo = "every " + $script:REPEAT_CHECK_AFTER_SECONDS + " seconds "
}

[string]$procInfo = ""
foreach ($process in $script:processesToDelete)
{
    $procInfo += " " + $process
}

""
"Checking " + $repeatInfo + "for unwanted processes:" + $procInfo
""

do
{
    # option "SilentlyContinue" tolerates search for not existing processes
    get-process $script:processesToDelete -ErrorAction SilentlyContinue `
        | foreach-object {$_.kill()}
    
    if ($script:REPEAT_CHECK_AFTER_SECONDS -gt 0)
    {
        # after some sleep time the check for the latest
        # results will be repeated
        Start-Sleep -Seconds $script:REPEAT_CHECK_AFTER_SECONDS
    }

} while ($script:REPEAT_CHECK_AFTER_SECONDS -gt 0)

